local ArkInventoryRules_MoreRules = ArkInventoryRules:NewModule ("ArkInventoryRules_MoreRules")

function ArkInventoryRules_MoreRules:OnEnable( )

	local registered
	
	registered = ArkInventoryRules.Register( self, "CLASSGEAR",	ArkInventoryRules_MoreRules.ClassGear ) -- Gear that is usable by the current class
	registered = ArkInventoryRules.Register( self, "CG", 		ArkInventoryRules_MoreRules.ClassGear ) -- Shorthand of ClassGear
	registered = ArkInventoryRules.Register( self, "COLLECTED",	ArkInventoryRules_MoreRules.Collected ) -- If the item appearance is known by the player
	registered = ArkInventoryRules.Register( self, "TMOG",		ArkInventoryRules_MoreRules.TMog ) -- If the item is even able to be transmogged
	registered = ArkInventoryRules.Register( self, "UPGRADE",	ArkInventoryRules_MoreRules.Upgrade ) -- Gear that is of a higher ilvl than what is worn
	registered = ArkInventoryRules.Register( self, "PLAYERUSE",	ArkInventoryRules_MoreRules.PlayerUse ) -- Items that are usable by the current player i.e. things not colored red
	registered = ArkInventoryRules.Register( self, "PU",	ArkInventoryRules_MoreRules.PlayerUse ) -- Shorthand of PlayerUse
	
end

local InventorySlots = {
    ['INVTYPE_HEAD'] = 1,
    ['INVTYPE_NECK'] = 2,
    ['INVTYPE_SHOULDER'] = 3,
    ['INVTYPE_BODY'] = 4,
    ['INVTYPE_CHEST'] = 5,
    ['INVTYPE_ROBE'] = 5,
    ['INVTYPE_WAIST'] = 6,
    ['INVTYPE_LEGS'] = 7,
    ['INVTYPE_FEET'] = 8,
    ['INVTYPE_WRIST'] = 9,
    ['INVTYPE_HAND'] = 10,
    ['INVTYPE_CLOAK'] = 15,
    ['INVTYPE_WEAPON'] = 16,
    ['INVTYPE_SHIELD'] = 17,
    ['INVTYPE_2HWEAPON'] = 16,
    ['INVTYPE_WEAPONMAINHAND'] = 16,
    ['INVTYPE_RANGED'] = 16,
    ['INVTYPE_RANGEDRIGHT'] = 16,
    ['INVTYPE_WEAPONOFFHAND'] = 17,
    ['INVTYPE_HOLDABLE'] = 17,
    -- ['INVTYPE_TABARD'] = 19,
}
 
local model = CreateFrame('DressUpModel')
function ArkInventoryRules_MoreRules.GetItemAppearance(itemLink)
    local itemID, _, _, slotName = GetItemInfoInstant(itemLink)
    if itemLink == itemID then
        itemLink = 'item:' .. itemID
    end
    local slot = InventorySlots[slotName]
    if not slot or not IsDressableItem(itemLink) then return end
    model:SetUnit('player')
    model:Undress()
    model:TryOn(itemLink, slot)
    local sourceID = model:GetSlotTransmogSources(slot)
    if sourceID then
        local categoryID, itemAppearanceID, canEnchant, texture, isCollected, itemLink = C_TransmogCollection.GetAppearanceSourceInfo(sourceID)
        return itemAppearanceID, isCollected, sourceID
    end
end
 
function ArkInventoryRules_MoreRules.PlayerHasAppearance(itemAppearanceID)
    local sources = C_TransmogCollection.GetAppearanceSources(itemAppearanceID)
    if sources then
        for i, source in pairs(sources) do
            if source.isCollected then
                return true
            end
        end
    end
    return false
end
 
function ArkInventoryRules_MoreRules.PlayerCanCollectAppearance(itemAppearanceID)
    local sources = C_TransmogCollection.GetAppearanceSources(itemAppearanceID)
    if sources then
        for i, source in pairs(sources) do
            if C_TransmogCollection.PlayerCanCollectSource(source.sourceID) then
                return true
            end
        end
    end
    return false
end
 
function ArkInventoryRules_MoreRules.CanTransmogItem(itemLink)
    local itemID = GetItemInfoInstant(itemLink)
    if itemID then
        local canBeChanged, noChangeReason, canBeSource, noSourceReason = C_Transmog.GetItemInfo(itemID)
        -- return canBeSource, noSourceReason
        if canBeSource then return true end
    end
    return false
end

function ArkInventoryRules_MoreRules.TMog( ... )
	local fn = "TMOG" -- Rule name for errors

		-- Verify that the item string information is loaded and not nil and that it is a valid item before continuing
	if not ArkInventoryRules.Object.h or ArkInventoryRules.Object.class ~= "item" then
		return false
	end

	local _, itemID, _, _, _, _, _, _, _, _, objequip = ArkInventory.ObjectInfo( ArkInventoryRules.Object.h )
	if itemID == nil or objequip == nil or objequip == "INVTYPE_TRINKET" or objequip == "INVTYPE_NECK" or objequip == "INVTYPE_FINGER" then return false end
	local itemAppearanceID, isCollected, sourceID = ArkInventoryRules_MoreRules.GetItemAppearance(itemID)
	
--	if itemID == nil or objequip == nil or objequip == "INVTYPE_TRINKET" or objequip == "INVTYPE_NECK" or objequip == "INVTYPE_FINGER" then return false end

	local _, _, quality = GetItemInfo(itemID)
	if quality == nil or quality <= 1 then return false end

	if not ArkInventoryRules_MoreRules.CanTransmogItem(itemID) then
		--ArkInventory.Output(string.lower( select( 3, ArkInventory.ObjectInfo( ArkInventoryRules.Object.h ) ) ) ..  " NOT TRANSMOGABLE" )
		return false
	else
		--ArkInventory.Output(string.lower( select( 3, ArkInventory.ObjectInfo( ArkInventoryRules.Object.h ) ) ) ..  " TRANSMOGABLE" )
		return true
	end

	-- Always return false in the end
	return false
end

function ArkInventoryRules_MoreRules.Collected( ... )
	-- Verify that the item string information is loaded and not nil and that it is a valid item before continuing
	local fn = "COLLECTED" -- Rule name for errors

	if not ArkInventoryRules.Object.h or ArkInventoryRules.Object.class ~= "item" then
		return false
	end

	local _, itemID, _, _, quality = ArkInventory.ObjectInfo( ArkInventoryRules.Object.h )
	if itemID == nil or quality == nil or quality <=1 then return false end

	local itemAppearanceID, isCollected, sourceID = ArkInventoryRules_MoreRules.GetItemAppearance(itemID)

	if itemAppearanceID == nil then return false end

	--if not ArkInventoryRules_MoreRules.CanTransmogItem(itemID) then
		--ArkInventory.Output(string.lower( select( 3, ArkInventory.ObjectInfo( ArkInventoryRules.Object.h ) ) ) ..  " NOT TRANSMOGABLE" )
	--	return false
	--end

	--if ArkInventoryRules_MoreRules.PlayerCanCollectAppearance(itemAppearanceID) then
	--	return false
	--end

	if ArkInventoryRules_MoreRules.PlayerHasAppearance(itemAppearanceID) then
		--ArkInventory.Output(string.lower( select( 3, ArkInventory.ObjectInfo( ArkInventoryRules.Object.h ) ) ) ..  " COLLECTED" )
		return true
	else
		--ArkInventory.Output(string.lower( select( 3, ArkInventory.ObjectInfo( ArkInventoryRules.Object.h ) ) ) ..  " NOT COLLECTED" )
		return false
	end

	-- Always return false in the end
	return false
end

function ArkInventoryRules_MoreRules.ClassGear( ... )
	local fn = "CLASSGEAR" -- Rule name for errors

	-- Verify that the item string information is loaded and not nil and that it is a valid item before continuing
	if not ArkInventoryRules.Object.h or ArkInventoryRules.Object.class ~= "item" then
		return false
	end
	
	local ac = select( '#', ... )

	-- Gather item information
	local _, itemID, _, _, _, _, _, _, _, _, objequip, _, _, _, objtype, objsubtype = ArkInventory.ObjectInfo( ArkInventoryRules.Object.h ) 
	if itemID == nil or  objequip == nil or objtype == nil or objsubtype == nil then return false end
	--ArkInventory.Output('itemID: ' .. itemID .. ' | objequip: ' .. ' | objtype: ' .. objtype .. ' | objsubtype: ' .. objsubtype)

	-- Is the item usable by the character?
	ArkInventory.TooltipSetHyperlink( ArkInventory.Global.Tooltip.Vendor, ArkInventoryRules.Object.h )
	if not ArkInventory.TooltipCanUse( ArkInventory.Global.Tooltip.Vendor ) then return false end

	-- Is the item something that can be equipped?
	local isEquippable = IsEquippableItem(itemID)
	if isEquippable == nil then return false end

	local class = ArkInventory.Global.Me.data.info.class
	
	-- Since we have filtered out any gear that isn't usable, all weapons left should now be usable, and trinkets
	if objtype == "Trinket" or objtype == "Weapon" then return true end
	
	if objtype == "Armor" then
		if objsubtype == "Miscellaneous" or
			objequip == "INVTYPE_CLOAK" or
			objequip == "INVTYPE_TRINKET" or
			objequip == "INVTYPE_NECK" or
			objequip == "INVTYPE_FINGER" or
			( ( class == "DEATHKNIGHT" ) and ( objsubtype == "Plate") ) or
			( ( class == "DEMONHUNTER" ) and ( objsubtype == "Leather") ) or
			( ( class == "DRUID" ) and ( objsubtype == "Leather") ) or
			( ( class == "HUNTER" ) and ( objsubtype == "Mail") ) or
			( ( class == "MAGE" ) and ( objsubtype == "Cloth") ) or
			( ( class == "MONK" ) and ( objsubtype == "Leather") ) or
			( ( class == "PALADIN" ) and ( objsubtype == "Plate") ) or
			( ( class == "PALADIN" ) and ( objsubtype == "Shields") ) or
			( ( class == "PRIEST" ) and ( objsubtype == "Cloth") ) or
			( ( class == "ROGUE" ) and ( objsubtype == "Leather") ) or
			( ( class == "SHAMAN" ) and ( objsubtype == "Mail") ) or
			( ( class == "SHAMAN" ) and ( objsubtype == "Shields") ) or
			( ( class == "WARLOCK" ) and ( objsubtype == "Cloth") ) or
			( ( class == "WARRIOR" ) and ( objsubtype == "Plate") ) or
			( ( class == "WARRIOR" ) and ( objsubtype == "Shields") ) then
			return true 
		end
	end

	-- Always return false in the end
	return false
end

function ArkInventoryRules_MoreRules.Upgrade( ... )
	local fn = "UPGRADE" -- Rule name for errors

	-- Verify that the item string information is loaded and not nil and that it is a valid item before continuing
	if not ArkInventoryRules.Object.h or ArkInventoryRules.Object.class ~= "item" then
		return false
	end

	local ac = select( '#', ... )

	local _, itemID, _, _, _, ilvl, _, _, _, _, _, _, _, _, objtype = ArkInventory.ObjectInfo( ArkInventoryRules.Object.h )
	if itemID == nil then return false end

	-- ClassGear check so armor types lower do not show up here. At top so that it doesn't pull object info if the item isn't even valid for upgrade checks
	if not ArkInventoryRules_MoreRules.ClassGear(ArkInventoryRules.Object.h) then return false end

	-- Is the item usable by the character?
	ArkInventory.TooltipSetHyperlink( ArkInventory.Global.Tooltip.Vendor, ArkInventoryRules.Object.h )
	if not ArkInventory.TooltipCanUse( ArkInventory.Global.Tooltip.Vendor ) then
		return false
	end

	-- Is the item something that can be equipped?
	local isEquippable = IsEquippableItem(itemID)
	if isEquippable == nil then return false end

	local name, _, _, _, _, _, _, _, _, _, _ = GetItemInfo(itemID) 
    local _, _, _, slotName = GetItemInfoInstant(itemID)
    local slot = InventorySlots[slotName]
    --ArkInventory.Output('SLOT ' .. name .. ' ' .. slotName .. ' ' .. iLevel )

    if slotName == "INVTYPE_FINGER" then
    	local finger0ilvl = select(6,ArkInventory.ObjectInfo(GetInventoryItemLink("player", 11)))
    	local finger1ilvl = select(6,ArkInventory.ObjectInfo(GetInventoryItemLink("player", 12)))
    	
    	if finger0ilvl == nil or finger1ilvl == nil then return true end
    	--ArkInventory.Output('FINGER ' .. finger0ilvl )
    	--ArkInventory.Output('FINGER ' .. name .. ' ' .. slotName .. ' ' .. ilvl )
    	
    	if ilvl >= finger0ilvl or ilvl >= finger1ilvl then return true end
    	
    	return false
    end

    if slotName == "INVTYPE_TRINKET" then
    	local trinket0ilvl = select(6,ArkInventory.ObjectInfo(GetInventoryItemLink("player", 13)))
    	local trinket1ilvl = select(6,ArkInventory.ObjectInfo(GetInventoryItemLink("player", 14)))
    	
    	if trinket0ilvl == nil or trinket1ilvl == nil then return true end
    	--ArkInventory.Output('TRINKET ' .. trinket0ilvl )
    	--ArkInventory.Output('TRINKET' .. name .. ' ' .. slotName .. ' ' .. ilvl )
    	
    	if ilvl >= trinket0ilvl or ilvl >= trinket1ilvl then return true end
    	
    	return false
    end

	if not objtype == "Armor" or not objtype == "Weapon" then return false end

	if slotName == nil or slot == nil then 
		return false
	else
		-- Take into account 2 handed weapons or ranged weapons when comparing offhand gear
		local eqSlotName = select(11,ArkInventory.ObjectInfo(GetInventoryItemLink("player", 16)))
		if eqSlotName == "INVTYPE_2HWEAPON" or eqSlotName == "INVTYPE_RANGED" then
			if slot == 17 then slot = 16 end
		end
		
		local eqilvl = select(6,ArkInventory.ObjectInfo(GetInventoryItemLink("player", slot)))
		

		if eqilvl == nil or ilvl >= eqilvl then return true end

	-- Always return false in the end
	end
	return false
end


function ArkInventoryRules_MoreRules.PlayerUse( ... )
	local fn = "PLAYERUSE" -- Rule name for errors

		-- Verify that the item string information is loaded and not nil and that it is a valid item before continuing
	if not ArkInventoryRules.Object.h or ArkInventoryRules.Object.class ~= "item" then
		return false
	end

	local itemID = select(2,ArkInventory.ObjectInfo( ArkInventoryRules.Object.h ) )
	if itemID == nil then return false end

	ArkInventory.TooltipSetHyperlink( ArkInventory.Global.Tooltip.Vendor, ArkInventoryRules.Object.h )
	if ArkInventory.TooltipCanUse( ArkInventory.Global.Tooltip.Vendor ) then
		return true
	end

	-- Always return false in the end
	return false
end

